<?php

/**
 * @internal
 * @author Mariusz Chwalba <koder@mediasystems.pl>
 * @copyright 2009 Media Systems
 * @package cashbill
 *
 * Obsługa prostego logowania do plików wszystkich przychodzących wiadomości
 * i raportów.
 *
 */
/**
 * @page diagnostic Diagnostyka i testy
 *
 * @section Logger plikowy
 * 
 * W ramach niniejszego pakietu aplikacji dołączony został gotowy mechanizm diagnostyczny,
 * umożliwiający zlokalizowanie większości błędów i problemów w procesie wdrażania usługi.
 * 
 * Aby uruchomić loggery diagnostyczne wystarczy po inicjalizacji bramki SMS dodać:
 * 
 * @code
 * 
 * $smsGateway->addReportHandler  ( new cReportLogger()   );
 * $smsGateway->addSMSHandler     ( new cSMSLogger()      );
 * $smsGateway->addSMSErrorHandler( new cSMSErrorLogger() );
 * 
 * @endcode
 *
 * Każdy z loggerów przyjmuje opcjonalny parametr w postaci ścieżki do pliku, w którym
 * powinien notować infrormacje. W przypadku pominięcia ścieżki, przyjmowany jest domyślny
 * plik logu: log/sms.log.
 * 
 */

class cLoggerException extends Exception
{
	/**
	 * Brak możliwości zapisu do pliku logowania.
	 */
	const CANNOT_WRITE_LOG = 100;
}

/**
 * @internal Prosty logger oparty o pliki.
 *
 */
class cSimpleLogger
{
	public $logFile;
	protected $fh;
	
	/**
	 * Uruchomienie loggera
	 *
	 * @param string $logFile - nazwa pliku zawierająca log. W przypadku braku będzie zapisywać do log/sms.log
	 */
	public function __construct( $logFile=null )
	{
		if ( $logFile )
		{
			$this->logFile = $logFile;
		}
		else
		{
			$this->logFile = dirname( __FILE__ ).'/log/sms.log';
		}
		
		touch($this->logFile);
		if ( !is_writable( $this->logFile ) ) throw new cLoggerException( 'Cannot write logfile: '.$this->logFile, cLoggerException::CANNOT_WRITE_LOG );
		
	}
	
	protected function microDate( $format, $date )
	{
		$micro = $date - floor( $date );
		return date( $format, floor( $date ) ).'.'.substr( round($micro,3), 2 );
	}
	
	
	protected function log( $message, $timeStamp=null )
	{
		$msg = '['.$this->microDate('Y-m-d H:i:s', microtime(true) ).( $timeStamp ? '/'.$this->microDate('Y-m-d H:i:s', $timeStamp ) : '' ).']'."\t".$message."\n";
		$this->write( $msg );
	}
	
	protected function write( $msg )
	{
		if ( !$this->fh ) $this->fh = fopen( $this->logFile, 'a' );
		fwrite( $this->fh, $msg );
	}
}

/**
 * Logger plikowy do wiadomości SMS.
 *
 * Gotowy handler obsługi wiadomości, zapisujący szczegóły każdej otrzymanej wiadomości
 * do pliku.
 * 
 * Dodłączenie do istniejącej obsługi sprowadza się do dodania polecenia:
 * $gateway->addSMSHandler( new cSMSLogger( [opcjonalna nazwa pliku] ) );
 * 
 * Zapisuje wszystkie szczegóły otrzymanych wiadomości do pliku.
 * Klasa wykorzystywana w celach diagnostycznych.
 * 
 * Po dodaniu klasy do bramki cGateway (metodą addSMSHandler)
 * wszystkie wiadomości odebrane przez bramkę zostawią odpowiednie
 * wpisy we wskazanym logu. 
 * 
 */
class cSMSLogger extends cSimpleLogger implements iSMSHandler 
{
	public function handle( cMessageSMSIn $sms )
	{
		$this->log( "Received message ID:{$sms->id} ({$sms->recivedParts}/{$sms->numberOfParts}), LA:{$sms->la}, msisdn:{$sms->msisdn}, prefix:{$sms->prefix}, text:{$sms->text}", $sms->time );
	}
}

/**
 * Logger plikowy do błędów wiadomości SMS.
 * 
 * Zapisuje wszystkie szczegóły o błędach otrzymanych wiadomości do pliku.
 * 
 * Po dodaniu klasy do bramki cGateway (metodą addSMSErrorHandler)
 * wszystkie błędnie odebrane wiadomości przez bramkę zostawią odpowiednie
 * wpisy we wskazanym logu. 
 * 
 */
class cSMSErrorLogger extends cSimpleLogger implements iSMSErrorHandler 
{
	public function handle( cMessageSMSIn $sms, $error )
	{
		$this->log( "ERROR {$error} message ID:{$sms->id} ({$sms->recivedParts}/{$sms->numberOfParts}), LA:{$sms->la}, msisdn:{$sms->msisdn}, prefix:{$sms->prefix}, text:{$sms->text}", $sms->time );
	}
}


/**
 * Logger plikowy do raportów SMS
 * 
 * Zapisuje wszystkie szczegóły raportów do pliku
 * 
 * Po dodaniu klasy do bramki cGateway (metodą addSMSReportHandler)
 * wszystkie raporty o dostarczeniu wiadomości zostaną zapisane we
 * wskazanym pliku. Pełny opis znaczenia raportów znajduje się
 * w załączniku "graf przejść stanów wiadomości", oraz w dokumentacji
 * klasy cMessageReport.
 *
 */
class cReportLogger extends cSimpleLogger implements iReportHandler 
{
	public function handle( cMessageReport $report )
	{
		$this->log( "Received report ID:{$report->id} ({$report->partNumber}/{$report->partNumber}), status:{$report->status}, detail:{$report->errorstr}", $report->time );		
	}	
}

?>